@@ -7,7 +7,7 @@ RDF::Trine::Parser::TriG - TriG RDF Parser
 
 =head1 VERSION
 
-This document describes RDF::Trine::Parser::TriG version 1.004
+This document describes RDF::Trine::Parser::TriG version 1.005
 
 =head1 SYNOPSIS
 
@@ -41,7 +41,7 @@ use RDF::Trine qw(literal);
 
 our ($VERSION);
 BEGIN {
-	$VERSION				= '1.004';
+	$VERSION				= '1.005';
 	$RDF::Trine::Parser::parser_names{ 'trig' }	= __PACKAGE__;
 	foreach my $ext (qw(trig)) {
 		$RDF::Trine::Parser::file_extensions{ $ext }	= __PACKAGE__;
@@ -75,35 +75,34 @@ sub _statement {
 	my $l		= shift;
 	my $t		= shift;
 	my $type	= $t->type;
-	given ($type) {
-		when (LBRACE) { return $self->_graph($l, $t); }
-		when (EQUALS) { return $self->_graph($l, $t); }
-		when (IRI) { return $self->_graph($l, $t); }
-		when (PREFIXNAME) { return $self->_graph($l, $t); }
-		when (WS) {}
-		when (PREFIX) {
-			$t	= $self->_get_token_type($l, PREFIXNAME);
-			my $name	= $t->value;
-			$t	= $self->_get_token_type($l, IRI);
-			my $iri	= $t->value;
-			$t	= $self->_get_token_type($l, DOT);
-			$self->{map}->add_mapping( $name => $iri );
-			if (my $ns = $self->{namespaces}) {
-				unless ($ns->namespace_uri($name)) {
-					$ns->add_mapping( $name => $iri );
-				}
+	if ($type == LBRACE) { return $self->_graph($l, $t); }
+	elsif ($type == EQUALS) { return $self->_graph($l, $t); }
+	elsif ($type == IRI) { return $self->_graph($l, $t); }
+	elsif ($type == PREFIXNAME) { return $self->_graph($l, $t); }
+	elsif ($type == WS) {}
+	elsif ($type == PREFIX) {
+		$t	= $self->_get_token_type($l, PREFIXNAME);
+		my $name	= $t->value;
+		$name		=~ s/:$//;
+		$t	= $self->_get_token_type($l, IRI);
+		my $iri	= $t->value;
+		$t	= $self->_get_token_type($l, DOT);
+		$self->{map}->add_mapping( $name => $iri );
+		if (my $ns = $self->{namespaces}) {
+			unless ($ns->namespace_uri($name)) {
+				$ns->add_mapping( $name => $iri );
 			}
 		}
-		when (BASE) {
-			$t	= $self->_get_token_type($l, IRI);
-			my $iri	= $t->value;
-			$t	= $self->_get_token_type($l, DOT);
-			$self->{baseURI}	= $iri;
-		}
-		default {
-			$self->_triple( $l, $t );
-			$t	= $self->_get_token_type($l, DOT);
-		}
+	}
+	elsif ($type == BASE) {
+		$t	= $self->_get_token_type($l, IRI);
+		my $iri	= $t->value;
+		$t	= $self->_get_token_type($l, DOT);
+		$self->{baseURI}	= $iri;
+	}
+	else {
+		$self->_triple( $l, $t );
+		$t	= $self->_get_token_type($l, DOT);
 	}
 }
 
@@ -125,7 +124,7 @@ sub _graph {
 	}
 	
 	if ($t->type != LBRACE) {
-		$self->throw_error("Expecting LBRACE but got " . decrypt_constant($type), $t, $l);
+		$self->_throw_error("Expecting LBRACE but got " . decrypt_constant($type), $t, $l);
 	}
 	
 	$t		= $self->_next_nonws($l);
@@ -188,7 +187,7 @@ sub _triple {
 			$self->_assert_list($subj, @objects);
 		}
 	} elsif (not($type==IRI or $type==PREFIXNAME or $type==BNODE)) {
-		$self->throw_error("Expecting resource or bnode but got " . decrypt_constant($type), $t, $l);
+		$self->_throw_error("Expecting resource or bnode but got " . decrypt_constant($type), $t, $l);
 	} else {
 		$subj	= $self->_token_to_node($t);
 	}
