@@ -7,7 +7,7 @@ RDF::Trine::Parser::Turtle - Turtle RDF Parser
 
 =head1 VERSION
 
-This document describes RDF::Trine::Parser::Turtle version 1.004
+This document describes RDF::Trine::Parser::Turtle version 1.005
 
 =head1 SYNOPSIS
 
@@ -44,7 +44,7 @@ use RDF::Trine::Parser::Turtle::Token;
 
 our $VERSION;
 BEGIN {
-	$VERSION				= '1.004';
+	$VERSION				= '1.005';
 	foreach my $ext (qw(ttl)) {
 		$RDF::Trine::Parser::file_extensions{ $ext }	= __PACKAGE__;
 	}
@@ -85,6 +85,7 @@ sub parse {
 	my $self	= shift;
 	local($self->{baseURI})	= shift;
 	my $string				= shift;
+# 	warn 'parse() content: ' . Dumper($string);	# XXX
 	local($self->{handle_triple}) = shift;
 	require Encode;
 	$string = Encode::encode("utf-8", $string);
@@ -178,7 +179,10 @@ sub _get_token_type {
 	my $l		= shift;
 	my $type	= shift;
 	my $t		= $self->_next_nonws($l);
-	return unless ($t);
+	unless ($t) {
+		$l->_throw_error(sprintf("Expecting %s but got EOF", decrypt_constant($type)));
+		return;
+	}
 	unless ($t->type eq $type) {
 		$self->_throw_error(sprintf("Expecting %s but got %s", decrypt_constant($type), decrypt_constant($t->type)), $t, $l);
 	}
@@ -190,32 +194,40 @@ sub _statement {
 	my $l		= shift;
 	my $t		= shift;
 	my $type	= $t->type;
-	given ($type) {
 # 		when (WS) {}
-		when (PREFIX) {
-			$t	= $self->_get_token_type($l, PREFIXNAME);
-			my $name	= $t->value;
-			$t	= $self->_get_token_type($l, IRI);
-			my $iri	= $t->value;
-			$t	= $self->_get_token_type($l, DOT);
-			$self->{map}->add_mapping( $name => $iri );
-			if (my $ns = $self->{namespaces}) {
-				unless ($ns->namespace_uri($name)) {
-					$ns->add_mapping( $name => $iri );
-				}
+	if ($type eq PREFIX) {
+		$t	= $self->_get_token_type($l, PREFIXNAME);
+		my $name	= $t->value;
+		$name		=~ s/:$//;
+		$t	= $self->_get_token_type($l, IRI);
+		my $r	= RDF::Trine::Node::Resource->new($t->value, $self->{baseURI});
+		my $iri	= $r->uri_value;
+		$t	= $self->_next_nonws($l);
+		if ($t and $t->type != DOT) {
+			$self->_unget_token($t);
+		}
+		$self->{map}->add_mapping( $name => $iri );
+		if (my $ns = $self->{namespaces}) {
+			unless ($ns->namespace_uri($name)) {
+				$ns->add_mapping( $name => $iri );
 			}
 		}
-		when (BASE) {
-			$t	= $self->_get_token_type($l, IRI);
-			my $iri	= $t->value;
-			$t	= $self->_get_token_type($l, DOT);
-			$self->{baseURI}	= $iri;
-		}
-		default {
-			$self->_triple( $l, $t );
-			$t	= $self->_get_token_type($l, DOT);
-		}
 	}
+	elsif ($type eq BASE) {
+		$t	= $self->_get_token_type($l, IRI);
+		my $r	= RDF::Trine::Node::Resource->new($t->value, $self->{baseURI});
+		my $iri	= $r->uri_value;
+		$t	= $self->_next_nonws($l);
+		if ($t and $t->type != DOT) {
+			$self->_unget_token($t);
+		}
+		$self->{baseURI}	= $iri;
+	}
+	else {
+		$self->_triple( $l, $t );
+		$t	= $self->_get_token_type($l, DOT);
+	}
+# 	}
 }
 
 sub _triple {
@@ -225,7 +237,9 @@ sub _triple {
 	my $type	= $t->type;
 	# subject
 	my $subj;
+	my $bnode_plist	= 0;
 	if ($type == LBRACKET) {
+		$bnode_plist	= 1;
 		$subj	= RDF::Trine::Node::Blank->new();
 		my $t	= $self->_next_nonws($l);
 		if ($t->type != RBRACKET) {
@@ -256,10 +270,19 @@ sub _triple {
 	} else {
 		$subj	= $self->_token_to_node($t);
 	}
-# 	warn "Subject: $subj\n";
+# 	warn "Subject: $subj\n";	# XXX
 	
-	#predicateObjectList
-	$self->_predicateObjectList($l, $subj);
+	if ($bnode_plist) {
+		#predicateObjectList?
+		$t	= $self->_next_nonws($l);
+		$self->_unget_token($t);
+		if ($t->type != DOT) {
+			$self->_predicateObjectList($l, $subj);
+		}
+	} else {
+		#predicateObjectList
+		$self->_predicateObjectList($l, $subj);
+	}
 }
 
 sub _assert_list {
@@ -292,7 +315,13 @@ sub _predicateObjectList {
 		$t		= $self->_next_nonws($l);
 		last unless ($t);
 		if ($t->type == SEMICOLON) {
+			my $sc	= $t;
+SEMICOLON_REPEAT:			
 			$t		= $self->_next_nonws($l);
+			unless ($t) {
+				$l->_throw_error("Expecting token after semicolon, but got EOF");
+			}
+			goto SEMICOLON_REPEAT if ($t->type == SEMICOLON);
 			if ($t->type == IRI or $t->type == PREFIXNAME or $t->type == A) {
 				next;
 			} else {
@@ -311,6 +340,7 @@ sub _objectList {
 	my $l		= shift;
 	my $subj	= shift;
 	my $pred	= shift;
+# 	warn "objectList: " . Dumper($subj, $pred);	# XXX
 	while (1) {
 		my $t		= $self->_next_nonws($l);
 		last unless ($t);
@@ -318,7 +348,7 @@ sub _objectList {
 		$self->_assert_triple($subj, $pred, $obj);
 		
 		$t	= $self->_next_nonws($l);
-		if ($t->type == COMMA) {
+		if ($t and $t->type == COMMA) {
 			next;
 		} else {
 			$self->_unget_token($t);
@@ -382,10 +412,10 @@ sub _object {
 			}
 			$self->_assert_list($obj, @objects);
 		}
-	} elsif (not($type==IRI or $type==PREFIXNAME or $type==A or $type==STRING1D or $type==STRING3D or $type==BNODE or $type==INTEGER or $type==DECIMAL or $type==DOUBLE or $type==BOOLEAN)) {
+	} elsif (not($type==IRI or $type==PREFIXNAME or $type==STRING1D or $type==STRING3D or $type==STRING1S or $type==STRING3S or $type==BNODE or $type==INTEGER or $type==DECIMAL or $type==DOUBLE or $type==BOOLEAN)) {
 		$self->_throw_error("Expecting object but got " . decrypt_constant($type), $t, $l);
 	} else {
-		if ($type==STRING1D or $type==STRING3D) {
+		if ($type==STRING1D or $type==STRING3D or $type==STRING1S or $type==STRING3S) {
 			my $value	= $t->value;
 			my $t		= $self->_next_nonws($l);
 			my $dt;
@@ -414,43 +444,45 @@ sub _token_to_node {
 	my $self	= shift;
 	my $t		= shift;
 	my $type	= shift || $t->type;
-	given ($type) {
-		when (A) {
-			return $rdf->type;
-		}
-		when (IRI) {
-			return RDF::Trine::Node::Resource->new($t->value, $self->{baseURI});
-		}
-		when (INTEGER) {
-			return RDF::Trine::Node::Literal->new($t->value, undef, $xsd->integer);
-		}
-		when (DECIMAL) {
-			return RDF::Trine::Node::Literal->new($t->value, undef, $xsd->decimal);
-		}
-		when (DOUBLE) {
-			return RDF::Trine::Node::Literal->new($t->value, undef, $xsd->double);
-		}
-		when (BOOLEAN) {
-			return RDF::Trine::Node::Literal->new($t->value, undef, $xsd->boolean);
-		}
-		when (PREFIXNAME) {
-			my ($ns, $local)	= @{ $t->args };
-			my $prefix			= $self->{map}->namespace_uri($ns);
-			unless (blessed($prefix)) {
-				$self->_throw_error("Use of undeclared prefix '$ns'", $t);
-			}
-			my $iri				= $prefix->uri($local);
-			return $iri;
-		}
-		when (BNODE) {
-			return RDF::Trine::Node::Blank->new($t->value);
-		}
-		when (STRING1D) {
-			return RDF::Trine::Node::Literal->new($t->value);
-		}
-		default {
-			$self->_throw_error("Converting $type to node not implemented", $t);
+	if ($type eq A) {
+		return $rdf->type;
+	}
+	elsif ($type eq IRI) {
+		return RDF::Trine::Node::Resource->new($t->value, $self->{baseURI});
+	}
+	elsif ($type eq INTEGER) {
+		return RDF::Trine::Node::Literal->new($t->value, undef, $xsd->integer);
+	}
+	elsif ($type eq DECIMAL) {
+		return RDF::Trine::Node::Literal->new($t->value, undef, $xsd->decimal);
+	}
+	elsif ($type eq DOUBLE) {
+		return RDF::Trine::Node::Literal->new($t->value, undef, $xsd->double);
+	}
+	elsif ($type eq BOOLEAN) {
+		return RDF::Trine::Node::Literal->new($t->value, undef, $xsd->boolean);
+	}
+	elsif ($type eq PREFIXNAME) {
+		my ($ns, $local)	= @{ $t->args };
+		$ns		=~ s/:$//;
+		my $prefix			= $self->{map}->namespace_uri($ns);
+		unless (blessed($prefix)) {
+			$self->_throw_error("Use of undeclared prefix '$ns'", $t);
 		}
+		my $iri				= $prefix->uri($local);
+		return $iri;
+	}
+	elsif ($type eq BNODE) {
+		return RDF::Trine::Node::Blank->new($t->value);
+	}
+	elsif ($type eq STRING1D) {
+		return RDF::Trine::Node::Literal->new($t->value);
+	}
+	elsif ($type eq STRING1S) {
+		return RDF::Trine::Node::Literal->new($t->value);
+	}
+	else {
+		$self->_throw_error("Converting $type to node not implemented", $t);
 	}
 }
 
